// Copyright (C) 2018 Avatao.com Innovative Learning Kft.
// All Rights Reserved. See LICENSE file for details.

import { Component, OnInit } from '@angular/core';
import { WebSocketService } from '../services/websocket.service';
import { ConsoleContentCommand, RewriteContentCommand, ShowLiveLogsCommand } from '../message-types/console-commands';
import { config } from '../config';
import { ProcessLogService } from '../services/processlog.service';
import { LogMessage } from '../message-types/log-message';
import { CommandMessage } from '../message-types/command-message';

@Component({
  selector: 'app-console',
  templateUrl: './console.component.html',
  styleUrls: ['./console.component.scss']
})
export class ConsoleComponent implements OnInit {
  console_content: string = config.console.defaultContent;
  rewriteContentWithProcessLogsOnDeploy: string = config.console.rewriteContentWithProcessLogsOnDeploy;

  command_handlers = {
    'write':                                 this.writeHandler.bind(this),
    'read':                                  this.readHandler.bind(this),
    'showLiveLogs':                          this.showLiveLogsHandler.bind(this),
    'rewriteContentWithProcessLogsOnDeploy': this.rewriteContentWithProcessLogsOnDeployHandler.bind(this)
  };

  constructor(private webSocketService: WebSocketService,
              private processLogService: ProcessLogService) {}

  ngOnInit() {
    this.webSocketService.connect();
    this.webSocketService.observeKey<CommandMessage>('console').subscribe(
      (event) => this.command_handlers[event.data.command](event.data)
    );
    this.processLogService.newLogs.subscribe((data) => this.newLogsHandler(data));
  }

  writeHandler(data: ConsoleContentCommand) {
    this.setContent(data.content);
  }

  readHandler(data: ConsoleContentCommand) {
    this.sendContent(this.console_content);
  }

  newLogsHandler(logs: LogMessage) {
    if (this.rewriteContentWithProcessLogsOnDeploy !== '') {
      const log = logs[this.rewriteContentWithProcessLogsOnDeploy];
      if (log) {
        this.setContent(log);
      }
    }
  }

  showLiveLogsHandler(data: ShowLiveLogsCommand) {
    this.processLogService.showLiveLogs = data.value;
  }

  rewriteContentWithProcessLogsOnDeployHandler(data: RewriteContentCommand) {
    this.rewriteContentWithProcessLogsOnDeploy = data.value;
  }

  setContent(content: string) {
    this.console_content = content;
  }

  sendContent(content: string) {
    this.webSocketService.send('console', {
      'command': 'read',
      'content': content
    });
  }
}
